@@ -36,13 +36,12 @@ setGeneric("pmapFromTranscripts", signature=c("x", "transcripts"),
 ### differs from sort() in that "-" strand elements are returned 
 ### highest value to lowest.
 .orderElementsByTranscription <- function(x) {
-    original <- unlist(sapply(elementLengths(x), function(xx) 1:xx), 
-                       use.names=FALSE)
+    part <- PartitioningByWidth(x)
+    original <- sequence(elementNROWS(part))
     ## order by position
     gr <- unlist(x, use.names = FALSE)
-    idx <- order(togroup(x), start(gr))
+    idx <- order(togroup(part), start(gr))
     gr <- gr[idx]
-    part <- PartitioningByWidth(x)
     ## handle zero-width ranges
     pstart <- start(part)[width(part) != 0L]
     pend <- end(part)[width(part) != 0L]
@@ -61,7 +60,7 @@ setGeneric("pmapFromTranscripts", signature=c("x", "transcripts"),
 .listCumsumShifted <- function(x) {
     cs <- unlist(cumsum(x), use.names=FALSE)
     shifted <- c(0L, head(cs, -1))
-    shifted[start(PartitioningByWidth(elementLengths(x)))] <- 0L
+    shifted[start(PartitioningByWidth(elementNROWS(x)))] <- 0L
     shifted
 }
 
@@ -100,7 +99,7 @@ setGeneric("pmapFromTranscripts", signature=c("x", "transcripts"),
             xrange <- shift(xrange, shifted[txHits])
         }
         ## seqnames come from 'transcripts'
-        txGroup <- togroup(transcripts)[txHits]
+        txGroup <- togroup(PartitioningByWidth(transcripts))[txHits]
         GRanges(names(transcripts)[txGroup], 
                 xrange, strand(flat)[txHits],
                 DataFrame(xHits, transcriptsHits=txGroup))
@@ -129,7 +128,7 @@ setMethod("mapToTranscripts", c("GenomicRanges", "GRangesList"),
             stop ("'transcripts' must have names")
         if (ignore.strand) {
             strand(transcripts) <- "*"
-        } else if (any(elementLengths(runValue(strand(transcripts))) > 1)) {
+        } else if (any(elementNROWS(runValue(strand(transcripts))) > 1)) {
                 stop(paste0("when ignore.strand=TRUE all inner list elements",
                             "of 'transcripts' must be the same strand"))
         }
@@ -177,21 +176,31 @@ setMethod("mapToTranscripts", c("ANY", "TxDb"),
 setMethod("pmapToTranscripts", c("GenomicRanges", "GenomicRanges"), 
     function(x, transcripts, ignore.strand=FALSE) 
     {
-        grl <- splitAsList(transcripts, seq_along(transcripts))
+        grl <- as(transcripts, "List")
         pmapToTranscripts(x, grl, ignore.strand)
     }
 )
 
+setMethod("pmapToTranscripts", c("GRangesList", "GRangesList"),
+          function(x, transcripts, ignore.strand=FALSE) 
+          {
+              gr <- unlist(x, use.names=FALSE)
+              ans <- pmapToTranscripts(gr,
+                  transcripts[togroup(PartitioningByWidth(x))],
+                  ignore.strand)
+              reduce(relist(ans, x))
+          })
+
 setMethod("pmapToTranscripts", c("GenomicRanges", "GRangesList"), 
     function(x, transcripts, ignore.strand=FALSE) 
     {
         if (!length(x))
             return(GRanges())
         if (is.null(names(transcripts)))
-            stop ("'transcripts' must have names")
+            names(transcripts) <- as.character(seq_along(transcripts))
         if (ignore.strand) {
             strand(transcripts) <- "*"
-        } else if (!all(elementLengths(runLength(strand(transcripts))) == 1)) {
+        } else if (!all(elementNROWS(runLength(strand(transcripts))) == 1)) {
             stop(paste0("when ignore.strand=TRUE all inner list elements ",
                         "of 'transcripts' must be the same strand"))
         }
@@ -208,7 +217,8 @@ setMethod("pmapToTranscripts", c("GenomicRanges", "GRangesList"),
         ## map i-th elements
         hits <- findOverlaps(x, unlist(transcripts, use.names=FALSE), 
                              type="within", ignore.strand=ignore.strand)
-        ith <- queryHits(hits) == togroup(transcripts)[subjectHits(hits)]
+        ith <- queryHits(hits) ==
+               togroup(PartitioningByWidth(transcripts))[subjectHits(hits)]
         map <- .mapToTranscripts(x, transcripts, hits[ith], ignore.strand)
 
         ## non-hits
@@ -242,7 +252,7 @@ setMethod("pmapToTranscripts", c("GenomicRanges", "GRangesList"),
         xHits <- queryHits(hits)
         txHits <- subjectHits(hits)
 
-        ## We check strand in this helper because 'hits' was not constructed 
+        ## Check strand in this helper because 'hits' was not constructed 
         ## with findOverlaps() or any other strand-aware method.
         if (is(transcripts, "GRangesList"))
             txstrand <- unlist(runValue(strand(transcripts)), use.names=FALSE)
@@ -325,7 +335,7 @@ setMethod("mapFromTranscripts", c("GenomicRanges", "GRangesList"),
 
         if (ignore.strand) {
             strand(transcripts) <- "*"
-        } else if (!all(elementLengths(runLength(strand(transcripts))) == 1)) {
+        } else if (!all(elementNROWS(runLength(strand(transcripts))) == 1)) {
             stop(paste0("when ignore.strand=TRUE all inner list ",
                         "elements of 'transcripts' must be the same strand"))
         }
@@ -343,13 +353,14 @@ setMethod("mapFromTranscripts", c("GenomicRanges", "GRangesList"),
         match1 <- match(xNames, txNames)
         group0 <- splitAsList(seq_along(txNames), match0)
         group1 <- group0[match(na.omit(match1), names(group0))]
-        xHits <- rep(which(!is.na(match1)), elementLengths(group1))
+        xHits <- rep(which(!is.na(match1)), elementNROWS(group1))
         txHits <- unlist(group1, use.names=FALSE)
         if (!length(xHits <- na.omit(xHits)))
             stop ("none of 'names(x)' are in 'names(transcripts)'")
 
         ## construct Hits
-        hits <- Hits(xHits, txHits, length(x), length(transcripts))
+        hits <- Hits(xHits, txHits, length(x), length(transcripts),
+                     sort.by.query=TRUE)
         map <- .mapFromTranscripts(x, transcripts, hits, ignore.strand) 
         ## remove zero-width ranges 
         if (any(zeroWidth <- width(map) == 0L))
@@ -427,7 +438,7 @@ setMethod("pmapFromTranscripts", c("Ranges", "GRangesList"),
     {
         if (!length(x) || !length(transcripts))
             return(GRangesList())
-        if (!all(elementLengths(runLength(strand(transcripts))) == 1)) {
+        if (!all(elementNROWS(runLength(strand(transcripts))) == 1)) {
             stop(paste0("when ignore.strand=TRUE all inner list ",
                         "elements of 'transcripts' must have the same strand"))
         }
@@ -455,7 +466,8 @@ setMethod("pmapFromTranscripts", c("GenomicRanges", "GenomicRanges"),
         transcripts <- .pmap_recycle(transcripts, maxlen)
 
         ## map i-th elements
-        hits <- Hits(seq_along(x), seq_along(x), length(x), length(x))
+        hits <- Hits(seq_along(x), seq_along(x), length(x), length(x),
+                     sort.by.query=TRUE)
         .mapFromTranscripts(x, transcripts, hits, ignore.strand)
     }
 )
@@ -469,7 +481,7 @@ setMethod("pmapFromTranscripts", c("GenomicRanges", "GRangesList"),
 
         if (ignore.strand) {
             strand(transcripts) <- "*"
-        } else if (!all(elementLengths(runLength(strand(transcripts))) == 1)) {
+        } else if (!all(elementNROWS(runLength(strand(transcripts))) == 1)) {
             stop(paste0("when ignore.strand=TRUE all inner list ",
                         "elements of 'transcripts' must have the same strand"))
         }
@@ -480,7 +492,8 @@ setMethod("pmapFromTranscripts", c("GenomicRanges", "GRangesList"),
         transcripts <- .pmap_recycle(transcripts, maxlen)
 
         ## map i-th elements
-        hits <- Hits(seq_along(x), seq_along(x), length(x), length(x))
+        hits <- Hits(seq_along(x), seq_along(x), length(x), length(x),
+                     sort.by.query=TRUE)
         map <- .mapFromTranscripts(x, 
                                    .orderElementsByTranscription(transcripts), 
                                    hits, ignore.strand)
