@@ -4,10 +4,9 @@
 ###
 ### For people who want to tap BioMart.
 ### Typical use:
-###   txdb <- makeTxDbFromBiomart(biomart="ensembl",
-###                               dataset="hsapiens_gene_ensembl")
+###   txdb <- makeTxDbFromBiomart("hsapiens_gene_ensembl")
 ### Speed:
-###   - for biomart="ensembl" and dataset="hsapiens_gene_ensembl":
+###   - for biomart="ENSEMBL_MART_ENSEMBL" and dataset="hsapiens_gene_ensembl":
 ###       (1) download takes about 8 min.
 ###       (2) db creation takes about 60-65 sec.
 ###
@@ -18,9 +17,9 @@
 ###
 
 ### A thin wrapper to useMart() that checks the user-supplied arguments.
-.useMart2 <- function(biomart="ensembl",
+.useMart2 <- function(biomart="ENSEMBL_MART_ENSEMBL",
                       dataset="hsapiens_gene_ensembl",
-                      host="www.biomart.org",
+                      host="www.ensembl.org",
                       port=80)
 {
     ### Could be that the user got the 'biomart' and/or 'dataset' values
@@ -43,41 +42,41 @@
     useMart(biomart=biomart, dataset=dataset, host=host, port=port)
 }
 
-### TODO: Share this with normalization of 'vals' arg in the transcripts(),
+### TODO: Share this with normalization of 'filter' arg in the transcripts(),
 ### exons(), cds(), and genes() extractors.
-.normarg_filters <- function(filters)
+.normarg_filter <- function(filter)
 {
-    if (is.null(filters) || identical(filters, ""))
+    if (is.null(filter) || identical(filter, ""))
         return(setNames(list(), character(0)))
-    if (!is.list(filters))
-        stop("'filters' must be a named list")
-    if (length(filters) == 0L)
+    if (!is.list(filter))
+        stop("'filter' must be a named list")
+    if (length(filter) == 0L)
         return(setNames(list(), character(0)))
-    filters_names <- names(filters)
-    if (is.null(filters_names))
-        stop("'filters' must be a named list")
-    if (any(filters_names %in% c(NA, "")))
-        stop("names on 'filters' cannot be NA or the empty string")
-    if (anyDuplicated(filters_names))
-        stop("names on 'filters' must be unique")
-    if (!all(sapply(filters, is.atomic)))
-        stop("'filters' list elements must be atomic")
-    if (any(sapply(filters, anyNA)))
-        stop("'filters' list elements cannot contain NAs")
-    filters
+    filter_names <- names(filter)
+    if (is.null(filter_names))
+        stop("'filter' must be a named list")
+    if (any(filter_names %in% c(NA, "")))
+        stop("names on 'filter' cannot be NA or the empty string")
+    if (anyDuplicated(filter_names))
+        stop("names on 'filter' must be unique")
+    if (!all(sapply(filter, is.atomic)))
+        stop("'filter' list elements must be atomic")
+    if (any(sapply(filter, anyNA)))
+        stop("'filter' list elements cannot contain NAs")
+    filter
 }
 
-### A thin wrapper to getBM() that takes the filters in the form of a named
+### A thin wrapper around getBM() that takes the filters in the form of a named
 ### list.
-.getBM2 <- function(attributes, filters=NULL, ...)
+.getBM2 <- function(attributes, filter=NULL, ...)
 {
-    filters <- .normarg_filters(filters)
-    if (length(filters) == 0L) {
+    filter <- .normarg_filter(filter)
+    if (length(filter) == 0L) {
         bm_filters <- bm_values <- ""
     } else {
-        bm_filters <- names(filters)
-        bm_values <- unname(filters)
-        bm_values[elementLengths(bm_values) == 0L] <- paste0(
+        bm_filters <- names(filter)
+        bm_values <- unname(filter)
+        bm_values[elementNROWS(bm_values) == 0L] <- paste0(
             "____this_is_a_very_unlikely_valid_value_but_you_never_know_",
             "this_is_just_a_dirty_hack_to_work_around_getBM_",
             "misinterpretation_of_empty_list_elements_in_values____")
@@ -93,28 +92,27 @@
 }
 
 ### Add filter created from user-supplied transcript_ids to user-specified
-### filters.
-.add_tx_id_filter <- function(filters,
-                              transcript_ids=NULL, id_prefix="ensembl_")
+### filter.
+.add_tx_id_filter <- function(filter, transcript_ids=NULL, id_prefix="ensembl_")
 {
-    filters <- .normarg_filters(filters)
+    filter <- .normarg_filter(filter)
     tx_name_colname <- paste0(id_prefix, "transcript_id")
     if (is.null(transcript_ids)) {
-        if (tx_name_colname %in% names(filters))
+        if (tx_name_colname %in% names(filter))
             warning(wmsg("transcript ids should be specified via the ",
-                         "'transcript_ids' rather than the 'filters' argument"))
-            return(filters)
+                         "'transcript_ids' rather than the 'filter' argument"))
+            return(filter)
     }
     if (!is.character(transcript_ids))
         stop("'transcript_ids' must ba a character vector")
     if (any(is.na(transcript_ids)))
         stop("'transcript_ids' cannot contain NAs")
-    if (tx_name_colname %in% names(filters))
+    if (tx_name_colname %in% names(filter))
         stop(wmsg("transcript ids cannot be specified via the ",
-                  "'transcript_ids' and 'filters' arguments ",
+                  "'transcript_ids' and 'filter' arguments ",
                   "at the same time"))
-    filters[[tx_name_colname]] <- transcript_ids
-    filters
+    filter[[tx_name_colname]] <- transcript_ids
+    filter
 }
 
 ## helper to extract the organism (as Genus and Species) from the dataset
@@ -147,13 +145,13 @@
 ### Download and preprocess the 'transcripts' data frame.
 ###
 
-.makeBiomartTranscripts <- function(filters, mart, transcript_ids,
+.makeBiomartTranscripts <- function(filter, mart, transcript_ids,
                                     recognized_attribs,
                                     id_prefix="ensembl_")
 {
     message("Download and preprocess the 'transcripts' data frame ... ",
             appendLF=FALSE)
-    bm_result <- .getBM2(recognized_attribs[['T']], filters,
+    bm_result <- .getBM2(recognized_attribs[['T']], filter,
                          mart=mart, bmHeader=FALSE)
     tx_name_colname <- paste0(id_prefix, "transcript_id")
     tx_name <- bm_result[[tx_name_colname]]
@@ -216,18 +214,27 @@
     biomart <- biomaRt:::martBM(mart)
     dataset <- biomaRt:::martDataset(mart)
     is_ensembl_mart <- tolower(substr(biomart, 1, 7)) == "ensembl"
-    is_plants_mart <- tolower(substr(biomart, 1, 12)) == "plants_mart_"
+    is_plants_mart <- tolower(substr(biomart, 1, 11)) == "plants_mart"
     if (is_ensembl_mart || is_plants_mart) {
         message("Download and preprocess the 'chrominfo' data frame ... ",
                 appendLF=FALSE)
         if (is_ensembl_mart) {
-            ## Ensembl mart
-            db_version <- .getBiomartDbVersion(mart, host, port, biomart)
-            ensembl_release <- .extractEnsemblReleaseFromDbVersion(db_version)
-            chromlengths <- try(fetchChromLengthsFromEnsembl(dataset,
-                                    release=ensembl_release,
-                                    extra_seqnames=extra_seqnames),
-                                silent=TRUE)
+            if (tolower(host) == "grch37.ensembl.org") {
+                ## Ensembl GRCh37 mart
+                chromlengths <- try(fetchChromLengthsFromEnsembl(dataset,
+                                        use.grch37=TRUE,
+                                        extra_seqnames=extra_seqnames),
+                                    silent=TRUE)
+            } else {
+                ## Ensembl mart
+                db_version <- .getBiomartDbVersion(mart, host, port, biomart)
+                ensembl_release <-
+                              .extractEnsemblReleaseFromDbVersion(db_version)
+                chromlengths <- try(fetchChromLengthsFromEnsembl(dataset,
+                                        release=ensembl_release,
+                                        extra_seqnames=extra_seqnames),
+                                    silent=TRUE)
+            }
         } else {
             ## Plants mart
             chromlengths <- try(fetchChromLengthsFromEnsemblPlants(dataset,
@@ -251,10 +258,10 @@
 }
 
 ## User-friendly wrapper to .makeBiomartChrominfo().
-getChromInfoFromBiomart <- function(biomart="ensembl",
+getChromInfoFromBiomart <- function(biomart="ENSEMBL_MART_ENSEMBL",
                                     dataset="hsapiens_gene_ensembl",
                                     id_prefix="ensembl_",
-                                    host="www.biomart.org",
+                                    host="www.ensembl.org",
                                     port=80)
 {
     mart <- .useMart2(biomart=biomart, dataset=dataset, host=host, port=port)
@@ -297,7 +304,8 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
     first_three_only <- total_nb_tx > 3L
     if (first_three_only)
         first_tx_names <- first_tx_names[1:3]
-    bm_result <- bm_result[tx_name %in% first_tx_names, , drop=FALSE]
+    bm_result <- S4Vectors:::extract_data_frame_rows(bm_result,
+                                     tx_name %in% first_tx_names)
     bm_result0 <- bm_result[-match(tx_name_colname, names(bm_result))]
     f <- factor(bm_result[[tx_name_colname]], levels=first_tx_names)
     first_tx_tables <- split(bm_result0, f)
@@ -308,9 +316,11 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
                         tx_table <- first_tx_tables[[i]]
                         if ("rank" %in% colnames(tx_table)) {
                             oo <- order(tx_table[["rank"]])
-                            tx_table <- tx_table[oo, , drop=FALSE]
+                            tx_table <-
+                              S4Vectors:::extract_data_frame_rows(tx_table, oo)
+                        } else {
+                            rownames(tx_table) <- NULL
                         }
-                        row.names(tx_table) <- NULL
                         subtitle <- paste0("  ", i, ". Transcript ",
                                            names(first_tx_tables)[i],
                                            ":")
@@ -584,7 +594,8 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
     ## More checking of the CDS of the "valid" transcripts ("valid" here means
     ## with no UTR anomalies).
     .check_cds(ans1[valid_tx_idx], cds_width[valid_tx_idx],
-               bm_result[valid_tx_idx, , drop=FALSE], id_prefix="ensembl_")
+               S4Vectors:::extract_data_frame_rows(bm_result, valid_tx_idx),
+               id_prefix="ensembl_")
     ans1
 }
 
@@ -602,7 +613,7 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
     ans
 }
 
-.makeBiomartSplicings <- function(filters, mart, transcripts_tx_id,
+.makeBiomartSplicings <- function(filter, mart, transcripts_tx_id,
                                   recognized_attribs, id_prefix="ensembl_")
 {
     ## Those are the strictly required fields.
@@ -621,7 +632,7 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
                         function(attribs) all(attribs %in% available_attribs))
     get_groups <- c("E1", names(has_group)[has_group])
     attributes <- unlist(recognized_attribs[get_groups], use.names=FALSE)
-    bm_result <- .getBM2(attributes, filters, mart=mart, bmHeader=FALSE)
+    bm_result <- .getBM2(attributes, filter, mart=mart, bmHeader=FALSE)
     tx_name_colname <- paste0(id_prefix, "transcript_id")
     tx_name <- bm_result[[tx_name_colname]]
     splicings_tx_id <- transcripts_tx_id[tx_name]
@@ -646,7 +657,7 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
 ### Download and preprocess the 'genes' data frame.
 ###
 
-.makeBiomartGenes <- function(filters, mart,
+.makeBiomartGenes <- function(filter, mart,
                               transcripts_tx_id, recognized_attribs,
                               id_prefix="ensembl_")
 {
@@ -654,7 +665,7 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
             appendLF=FALSE)
     attributes <- c(recognized_attribs[['G']],
                     paste0(id_prefix, "transcript_id"))
-    bm_result <- .getBM2(attributes, filters, mart=mart, bmHeader=FALSE)
+    bm_result <- .getBM2(attributes, filter, mart=mart, bmHeader=FALSE)
     tx_name_colname <- paste0(id_prefix, "transcript_id")
     gene_id_colname <- paste0(id_prefix, "gene_id")
     tx_name <- bm_result[[tx_name_colname]]
@@ -734,37 +745,33 @@ getChromInfoFromBiomart <- function(biomart="ensembl",
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### makeTxDbFromBiomart()
 ###
-### TODO: Rename the 'filters' arg -> 'filter' and set its default to NULL
-### so it's consistent with the 'vals' arg of the transcripts(), exons(),
-### cds(), and genes() extractors which we should also rename 'filter'.
-###
 
-makeTxDbFromBiomart <- function(biomart="ensembl",
+makeTxDbFromBiomart <- function(biomart="ENSEMBL_MART_ENSEMBL",
                                 dataset="hsapiens_gene_ensembl",
                                 transcript_ids=NULL,
                                 circ_seqs=DEFAULT_CIRC_SEQS,
-                                filters="",
+                                filter=NULL,
                                 id_prefix="ensembl_",
-                                host="www.biomart.org",
+                                host="www.ensembl.org",
                                 port=80,
                                 taxonomyId=NA,
                                 miRBaseBuild=NA)
 {
     mart <- .useMart2(biomart=biomart, dataset=dataset, host=host, port=port)
     id_prefix <- .normarg_id_prefix(id_prefix)
-    filters <- .add_tx_id_filter(filters, transcript_ids, id_prefix)
+    filter <- .add_tx_id_filter(filter, transcript_ids, id_prefix)
     valid_filter_names <- listFilters(mart, what="name")
-    invalid_filter_names <- setdiff(names(filters), valid_filter_names)
+    invalid_filter_names <- setdiff(names(filter), valid_filter_names)
     if (length(invalid_filter_names) != 0L) {
         in1string <- paste0(invalid_filter_names, collapse=", ")
         stop(wmsg("Invalid filter name(s): ", in1string,
                   "\n\nPlease use the listFilters() function from the ",
                   "biomaRt package to get valid filter names."))
     }
-    is_full_dataset <- length(filters) == 0L
+    is_full_dataset <- length(filter) == 0L
     recognized_attribs <- recognizedBiomartAttribs(id_prefix)
 
-    transcripts <- .makeBiomartTranscripts(filters, mart,
+    transcripts <- .makeBiomartTranscripts(filter, mart,
                                            transcript_ids,
                                            recognized_attribs,
                                            id_prefix)
@@ -776,9 +783,9 @@ makeTxDbFromBiomart <- function(biomart="ensembl",
                                        host, port)
     if (!is_full_dataset) {
         keep_idx <- which(chrominfo[ , "chrom"] %in% transcripts$tx_chrom)
-        chrominfo <- chrominfo[keep_idx, , drop=FALSE]
+        chrominfo <- S4Vectors:::extract_data_frame_rows(chrominfo, keep_idx)
     }
-    splicings <- .makeBiomartSplicings(filters, mart,
+    splicings <- .makeBiomartSplicings(filter, mart,
                                        transcripts_tx_id,
                                        recognized_attribs,
                                        id_prefix=id_prefix)
@@ -792,16 +799,18 @@ makeTxDbFromBiomart <- function(biomart="ensembl",
                     length(invalid_tx), " transcripts) ... ",
                     appendLF=FALSE)
             keep_idx1 <- !(transcripts$tx_id %in% invalid_tx)
-            transcripts <- transcripts[keep_idx1, ]
+            transcripts <- S4Vectors:::extract_data_frame_rows(transcripts,
+                                                               keep_idx1)
             transcripts_tx_id <- transcripts_tx_id[keep_idx1]
             keep_idx2 <- !(splicings$tx_id %in% invalid_tx)
-            splicings <- splicings[keep_idx2, , drop=FALSE]
+            splicings <- S4Vectors:::extract_data_frame_rows(splicings,
+                                                             keep_idx2)
             message("OK")
         }
         splicings$utr_anomaly <- NULL
     }
         
-    genes <- .makeBiomartGenes(filters, mart, transcripts_tx_id,
+    genes <- .makeBiomartGenes(filter, mart, transcripts_tx_id,
                                recognized_attribs, id_prefix)
     metadata <- .prepareBiomartMetadata(mart, is_full_dataset,
                                         host, port, taxonomyId, miRBaseBuild)
@@ -814,5 +823,3 @@ makeTxDbFromBiomart <- function(biomart="ensembl",
     txdb
 }
 
-makeTranscriptDbFromBiomart <- function(...) .Defunct("makeTxDbFromBiomart")
-
