@@ -20,7 +20,7 @@
                   "only supports 'ties.method=\"first\"' at the moment"))
     x_partitioning <- PartitioningByEnd(x)
     unlisted_x <- unlist(x, use.names=FALSE)
-    oo <- S4Vectors:::orderIntegerPairs(togroup(x_partitioning), unlisted_x)
+    oo <- orderIntegerPairs(togroup(x_partitioning), unlisted_x)
     unlisted_ans <- integer(length(oo))
     unlisted_ans[oo] <- seq_along(oo)
     ans <- relist(unlisted_ans, x_partitioning)
@@ -52,7 +52,9 @@ GTF_COLNAMES <- c("type", "gene_id", "transcript_id", "exon_id")
 .GENE_TYPES <- c("gene", "pseudogene", "transposable_element_gene")
 .TX_TYPES <- c("transcript", "pseudogenic_transcript", "primary_transcript",
                "mRNA", "ncRNA", "rRNA", "snoRNA", "snRNA", "tRNA", "tmRNA",
-               "miRNA", "miRNA_primary_transcript")
+               "miRNA", "miRNA_primary_transcript",
+               "RNase_P_RNA", "RNase_MRP_RNA", "SRP_RNA", "misc_RNA",
+               "antisense_RNA", "antisense")
 .EXON_TYPES <- c("exon", "pseudogenic_exon")
 .CDS_TYPES <- "CDS"
 .STOP_CODON_TYPES <- "stop_codon"
@@ -106,11 +108,11 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     transcript_id
 }
 
-### If we have "gene_id" and "transcript_id" metadata columns (and if they
-### don't contain only NAs) then we assume the GRanges object is in GTF format.
-### Otherwise we assume it's in GFF3 format.
-.is_gtf_format <- function(gene_id, transcript_id)
-    !(.no_id(gene_id) || .no_id(transcript_id))
+### If we have no "ID" metadata column but "gene_id" and "transcript_id"
+### metadata columns (and if they don't contain only NAs) then we assume the
+### GRanges object is in GTF format. Otherwise we assume it's in GFF3 format.
+.is_gtf_format <- function(ID, gene_id, transcript_id)
+    (.no_id(ID) && !.no_id(gene_id) && !.no_id(transcript_id))
 
 .get_ID <- function(gr_mcols, type, gene_id, transcript_id, gtf.format=FALSE)
 {
@@ -358,9 +360,9 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         return(integer(0))
     id_parts <- strsplit(id, "\\.|:")
     ## Fix non-sensical output of strsplit() on empty strings.
-    id_parts[elementLengths(id_parts) == 0L] <- ""
+    id_parts[elementNROWS(id_parts) == 0L] <- ""
     unlisted_id_parts <- unlist(id_parts, use.names=FALSE)
-    idx <- cumsum(elementLengths(id_parts))
+    idx <- cumsum(elementNROWS(id_parts))
     rank <- unlisted_id_parts[idx]
     rank <- suppressWarnings(as.integer(rank))
     if (any(is.na(rank)))
@@ -369,7 +371,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tmp <- unname(splitAsList(rank, parent_id))
     if (any(any(duplicated(tmp))))
         return(NULL)
-    if (!(all(min(tmp) == 1L) && all(max(tmp) == elementLengths(tmp))))
+    if (!(all(min(tmp) == 1L) && all(max(tmp) == elementNROWS(tmp))))
         return(NULL)
     rank
 }
@@ -379,19 +381,19 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 {
     chrom_by_tx <- split(Rle(exon_chrom), tx_id)
     tx_chrom <- runValue(chrom_by_tx)
-    bad_tx1 <- names(which(elementLengths(tx_chrom) > 1L))
+    bad_tx1 <- names(which(elementNROWS(tx_chrom) > 1L))
 
     strand_by_tx <- split(Rle(exon_strand), tx_id)
     tx_strand <- runValue(strand_by_tx)
-    is_bad <- elementLengths(tx_strand) > 1L
+    is_bad <- elementNROWS(tx_strand) > 1L
     bad_tx2 <- names(which(is_bad))
     tx_strand[is_bad] <- "*"
     minus_idx <- which(as.character(tx_strand) == "-")
 
     ex_by_tx <- split(IRanges(exon_start, exon_end), tx_id)
     reduced_ex_by_tx <- reduce(ex_by_tx, min.gapwidth=0L)
-    bad_tx3 <- names(which(elementLengths(reduced_ex_by_tx) !=
-                           elementLengths(ex_by_tx)))
+    bad_tx3 <- names(which(elementNROWS(reduced_ex_by_tx) !=
+                           elementNROWS(ex_by_tx)))
 
     start_by_tx <- start(ex_by_tx)
     start_by_tx[minus_idx] <- start_by_tx[minus_idx] * (-1L)
@@ -416,7 +418,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         stop(wmsg("some ", what, "s are mapped twice to the same transcript"))
 
     tx_id <- factor(unlist(exon_Parent, use.names=FALSE))
-    nparent_per_ex <- elementLengths(exon_Parent)
+    nparent_per_ex <- elementNROWS(exon_Parent)
     exon_id <- rep.int(ID[exon_IDX], nparent_per_ex)
     exon_name <- rep.int(Name[exon_IDX], nparent_per_ex)
     exon_chrom <- rep.int(seqnames(gr)[exon_IDX], nparent_per_ex)
@@ -481,7 +483,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 .add_missing_exons <- function(exons, transcripts)
 {
     is_noextx <- !(transcripts$tx_id %in% exons$tx_id)
-    missing_exons <- transcripts[is_noextx, , drop=FALSE]
+    missing_exons <- S4Vectors:::extract_data_frame_rows(transcripts, is_noextx)
     if (nrow(missing_exons) == 0L)
         return(exons)
     missing_exons$tx_type <- NULL
@@ -513,7 +515,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx_id <- names(transcripts_by_id)
 
     tx_name <- unique(transcripts_by_id[ , "tx_name"])
-    bad_tx <- names(which(elementLengths(tx_name) != 1L))
+    bad_tx <- names(which(elementNROWS(tx_name) != 1L))
     if (length(bad_tx) != 0L) {
         in1string <- paste0(sort(bad_tx), collapse=", ")
         stop(wmsg("The following transcripts have multiple parts that cannot ",
@@ -522,7 +524,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx_name <- as.character(tx_name)
 
     tx_type <- unique(transcripts_by_id[ , "tx_type"])
-    bad_tx <- names(which(elementLengths(tx_type) != 1L))
+    bad_tx <- names(which(elementNROWS(tx_type) != 1L))
     if (length(bad_tx) != 0L) {
         in1string <- paste0(sort(bad_tx), collapse=", ")
         stop(wmsg("The following transcripts have multiple parts that cannot ",
@@ -531,7 +533,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx_type <- as.character(tx_type)
 
     tx_chrom <- unique(transcripts_by_id[ , "tx_chrom"])
-    bad_tx <- names(which(elementLengths(tx_chrom) != 1L))
+    bad_tx <- names(which(elementNROWS(tx_chrom) != 1L))
     if (length(bad_tx) != 0L) {
         in1string <- paste0(sort(bad_tx), collapse=", ")
         stop(wmsg("The following transcripts have multiple parts that cannot ",
@@ -540,7 +542,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx_chrom <- as.character(tx_chrom)
 
     tx_strand <- unique(transcripts_by_id[ , "tx_strand"])
-    bad_tx <- names(which(elementLengths(tx_strand) != 1L))
+    bad_tx <- names(which(elementNROWS(tx_strand) != 1L))
     if (length(bad_tx) != 0L) {
         in1string <- paste0(sort(bad_tx), collapse=", ")
         stop(wmsg("The following transcripts have multiple parts that cannot ",
@@ -611,11 +613,11 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx_type <- rep.int("inferred_from_exons", length(tx_id))
 
     tx_chrom <- unique(exons_by_id[ , "exon_chrom"])
-    tx_chrom[elementLengths(tx_chrom) != 1L] <- NA_character_
+    tx_chrom[elementNROWS(tx_chrom) != 1L] <- NA_character_
     tx_chrom <- as.character(tx_chrom)
 
     tx_strand <- unique(exons_by_id[ , "exon_strand"])
-    tx_strand[elementLengths(tx_strand) != 1L] <- NA_character_
+    tx_strand[elementNROWS(tx_strand) != 1L] <- NA_character_
     tx_strand <- as.character(tx_strand)
 
     tx_start <- unname(min(exons_by_id[ , "exon_start"]))
@@ -636,7 +638,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 .add_missing_transcripts <- function(transcripts, exons)
 {
     is_orphan <- !(exons$tx_id %in% transcripts$tx_id)
-    orphan_exons <- exons[is_orphan, ]
+    orphan_exons <- S4Vectors:::extract_data_frame_rows(exons, is_orphan)
     missing_transcripts <- .infer_transcripts_from_exons(orphan_exons)
     rbind(transcripts, missing_transcripts)
 }
@@ -754,16 +756,17 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 
     ## First we try to find their parent via the Dbxref tag, if present.
     if (!is.null(Dbxref)) {
-        idx0 <- which(elementLengths(tx2genes) == 0L)
+        idx0 <- which(elementNROWS(tx2genes) == 0L)
         tx_Dbxref <- Dbxref[tx_IDX[idx0]]
         gene_Dbxref <- Dbxref[gene_IDX]
         tx_Dbxref_unlisted <- unlist(tx_Dbxref, use.names=FALSE)
         gene_Dbxref_unlisted <- unlist(gene_Dbxref, use.names=FALSE)
         hits <- findMatches(tx_Dbxref_unlisted, gene_Dbxref_unlisted)
-        hits <- remapHits(hits, query.map=togroup(tx_Dbxref),
-                                new.queryLength=length(tx_Dbxref),
-                                subject.map=togroup(gene_Dbxref),
-                                new.subjectLength=length(gene_Dbxref))
+        hits <- remapHits(hits,
+            Lnodes.remapping=S4Vectors:::quick_togroup(tx_Dbxref),
+            new.nLnode=length(tx_Dbxref),
+            Rnodes.remapping=S4Vectors:::quick_togroup(gene_Dbxref),
+            new.nRnode=length(gene_Dbxref))
         tx2genes[idx0] <- relist(ID[gene_IDX][subjectHits(hits)],
                                  as(hits, "PartitioningByEnd"))
     }
@@ -778,12 +781,12 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     ## Then, if we still have transcripts with no parent, we use the geneID
     ## tag (if present) to assign them an external gene id.
     if (!is.null(geneID)) {
-        idx0 <- which(elementLengths(tx2genes) == 0L)
+        idx0 <- which(elementNROWS(tx2genes) == 0L)
         tx2genes[idx0] <- geneID[tx_IDX[idx0]]
         tx2genes <- tx2genes[!is.na(tx2genes)]
     }
 
-    tx_id <- rep.int(tx_id, elementLengths(tx2genes))
+    tx_id <- rep.int(tx_id, elementNROWS(tx2genes))
     gene_id <- unlist(tx2genes, use.names=FALSE)
     data.frame(tx_id=tx_id, gene_id=gene_id, stringsAsFactors=FALSE)
 }
@@ -794,7 +797,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     ## Keep only unique (tx_id, gene_id) pairs.
     tx_id <- factor(transcript_id, levels=unique(transcript_id))
     gene_id <- factor(gene_id, levels=unique(gene_id))
-    keep_idx <- which(!S4Vectors:::duplicatedIntegerPairs(tx_id, gene_id))
+    keep_idx <- which(!duplicatedIntegerPairs(tx_id, gene_id))
     tx_id <- tx_id[keep_idx]
     gene_id <- gene_id[keep_idx]
 
@@ -845,10 +848,10 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         organism <- paste(org[1,2], org[1,3])
         idx <- match("Organism", metadata$name)
         if (!is.na(idx))
-            metadata <- metadata[-idx, , drop=FALSE]
+            metadata <- S4Vectors:::extract_data_frame_rows(metadata, -idx)
         idx <- match("Taxonomy ID", metadata$name)
         if (!is.na(idx))
-            metadata <- metadata[-idx, , drop=FALSE]
+            metadata <- S4Vectors:::extract_data_frame_rows(metadata, -idx)
         df2 <- data.frame(name=c("Organism", "Taxonomy ID"),
                           value=c(organism, taxonomyId),
                           stringsAsFactors=FALSE)
@@ -884,7 +887,7 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
     type <- .get_type(gr_mcols)
     gene_id <- .get_gene_id(gr_mcols)
     transcript_id <- .get_transcript_id(gr_mcols, gene_id, type)
-    gtf.format <- .is_gtf_format(gene_id, transcript_id)
+    gtf.format <- .is_gtf_format(gr_mcols$ID, gene_id, transcript_id)
     ID <- .get_ID(gr_mcols, type, gene_id, transcript_id,
                   gtf.format=gtf.format)
     Parent <- .get_Parent(gr_mcols, type, gene_id, transcript_id,
@@ -958,11 +961,16 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
     drop3_tx <- unique(drop3_tx)
     drop_tx <- unique(c(drop1_tx, drop2_tx, drop3_tx))
     if (length(drop_tx) != 0L) {
-        exons <- exons[!(exons$tx_id %in% drop_tx), ]
-        cds <- cds[!(cds$tx_id %in% drop_tx), ]
-        stop_codons <- stop_codons[!(stop_codons$tx_id %in% drop_tx), ]
-        transcripts <- transcripts[!(transcripts$tx_id %in% drop_tx), ]
-        genes <- genes[!(genes$tx_id %in% drop_tx), ]
+        exons <- S4Vectors:::extract_data_frame_rows(exons,
+                                     !(exons$tx_id %in% drop_tx))
+        cds <- S4Vectors:::extract_data_frame_rows(cds,
+                                     !(cds$tx_id %in% drop_tx))
+        stop_codons <- S4Vectors:::extract_data_frame_rows(stop_codons,
+                                     !(stop_codons$tx_id %in% drop_tx))
+        transcripts <- S4Vectors:::extract_data_frame_rows(transcripts,
+                                     !(transcripts$tx_id %in% drop_tx))
+        genes <- S4Vectors:::extract_data_frame_rows(genes,
+                                     !(genes$tx_id %in% drop_tx))
         if (length(drop1_tx) != 0L) {
             in1string <- paste0(sort(drop1_tx), collapse=", ")
             warning(wmsg(
@@ -998,9 +1006,12 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
     splicings <- .make_splicings(exons, cds, stop_codons)
     drop_tx <- .get_rejected_transcripts()
     if (length(drop_tx) != 0L) {
-        transcripts <- transcripts[!(transcripts$tx_id %in% drop_tx), ]
-        splicings <- splicings[!(splicings$tx_id %in% drop_tx), ]
-        genes <- genes[!(genes$tx_id %in% drop_tx), ]
+        transcripts <- S4Vectors:::extract_data_frame_rows(transcripts,
+                                           !(transcripts$tx_id %in% drop_tx))
+        splicings <- S4Vectors:::extract_data_frame_rows(splicings,
+                                         !(splicings$tx_id %in% drop_tx))
+        genes <- S4Vectors:::extract_data_frame_rows(genes,
+                                     !(genes$tx_id %in% drop_tx))
     }
 
     ## Turn the "tx_id" column in 'splicings', 'genes', and 'transcripts' into
@@ -1042,8 +1053,7 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
                 transcripts$tx_start, transcripts$tx_end,
                 transcripts$tx_strand,
                 transcripts$tx_name)
-    transcripts <- transcripts[oo, , drop=FALSE]
-    rownames(transcripts) <- NULL
+    transcripts <- S4Vectors:::extract_data_frame_rows(transcripts, oo)
 
     keep_cols <- c("exon_rank", "exon_name", "exon_chrom", "exon_strand",
                    "exon_start", "exon_end", "cds_start", "cds_end")
@@ -1052,8 +1062,7 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
                 splicings$exon_start, splicings$exon_end,
                 splicings$exon_strand,
                 splicings$exon_name)
-    splicings <- splicings[oo, , drop=FALSE]
-    rownames(splicings) <- NULL
+    splicings <- S4Vectors:::extract_data_frame_rows(splicings, oo)
 
     list(transcripts=transcripts, splicings=splicings)
 }
@@ -1086,9 +1095,8 @@ test_makeTxDbFromGRanges_on_Ensembl_organism_gtf <- function(organism)
     dataset <- paste0(substr(dataset[[1L]], 1L, 1L),
                       dataset[[2L]],
                       "_gene_ensembl")
-    cat("txdb2 <- makeTxDbFromBiomart(\"ensembl\", ",
-                                     "\"", dataset, "\") ... ", sep="")
-    txdb2 <- try(makeTxDbFromBiomart("ensembl", dataset))
+    cat("txdb2 <- makeTxDbFromBiomart(\"", dataset, "\") ... ", sep="")
+    txdb2 <- try(makeTxDbFromBiomart(dataset))
     if (is(txdb2, "try-error")) {
         cat("ABORT\n")
         return(NA)
