@@ -154,16 +154,24 @@ UCSCGenomeToOrganism <- function(genome){
   "wgEncodeGencodePolyaV3",           "Gencode Genes",     "Gencode PolyA",
 
   ## Tables/tracks specific to hg19.
+  "wgEncodeGencodeBasicV19",          "GENCODE Genes V19", NA,
+  "wgEncodeGencodeCompV19",           "GENCODE Genes V19", NA,
+  "wgEncodeGencodePseudoGeneV19",     "GENCODE Genes V19", NA,
+  "wgEncodeGencode2wayConsPseudoV19", "GENCODE Genes V19", NA,
+  "wgEncodeGencodePolyaV19",          "GENCODE Genes V19", NA,
+
   "wgEncodeGencodeBasicV17",          "GENCODE Genes V17", NA,
   "wgEncodeGencodeCompV17",           "GENCODE Genes V17", NA,
   "wgEncodeGencodePseudoGeneV17",     "GENCODE Genes V17", NA,
   "wgEncodeGencode2wayConsPseudoV17", "GENCODE Genes V17", NA,
   "wgEncodeGencodePolyaV17",          "GENCODE Genes V17", NA,
+
   "wgEncodeGencodeBasicV14",          "GENCODE Genes V14", NA,
   "wgEncodeGencodeCompV14",           "GENCODE Genes V14", NA,
   "wgEncodeGencodePseudoGeneV14",     "GENCODE Genes V14", NA,
   "wgEncodeGencode2wayConsPseudoV14", "GENCODE Genes V14", NA,
   "wgEncodeGencodePolyaV14",          "GENCODE Genes V14", NA,
+
   "wgEncodeGencodeBasicV7",           "GENCODE Genes V7",  NA,
   "wgEncodeGencodeCompV7",            "GENCODE Genes V7",  NA,
   "wgEncodeGencodePseudoGeneV7" ,     "GENCODE Genes V7",  NA,
@@ -179,25 +187,28 @@ UCSCGenomeToOrganism <- function(genome){
   "sgdGene",                          "SGD Genes",         NA
 )
 
-supportedUCSCtables <- function()
+supportedUCSCtables <- function(genome="hg19")
 {
+    if (!isSingleString(genome))
+        stop("'genome' must be a single string")
     mat <- matrix(.SUPPORTED_UCSC_TABLES, ncol=3, byrow=TRUE)
     colnames(mat) <- c("tablename", "track", "subtrack")
-    data.frame(track=mat[ , "track"], subtrack=mat[ , "subtrack"],
-               row.names=mat[ , "tablename"],
-               stringsAsFactors=FALSE)
+    ans <- data.frame(track=mat[ , "track"], subtrack=mat[ , "subtrack"],
+                      row.names=mat[ , "tablename"],
+                      stringsAsFactors=FALSE)
+    if (genome %in% c("hg17", "hg16", "mm8", "mm7", "rn3")) {
+        ans$track[rownames(ans) == "knownGene"] <- "Known Genes"
+    } else if (genome %in% "hg38") {
+        ans$track[rownames(ans) == "knownGene"] <- "GENCODE v22"
+    }
+    ans
 }
 
 .tablename2track <- function(tablename, genome)
 {
     if (!isSingleString(tablename))
         stop("'tablename' must be a single string")
-    if (!isSingleString(genome))
-        stop("'genome' must be a single string")
-    if (tablename == "knownGene"
-     && genome %in% c("hg17", "hg16", "mm8", "mm7", "rn3"))
-        return("Known Genes")
-    track <- supportedUCSCtables()[tablename, "track"]
+    track <- supportedUCSCtables(genome)[tablename, "track"]
     if (is.na(track))
         stop("UCSC table \"", tablename, "\" is not supported")
     track
@@ -524,15 +535,15 @@ supportedUCSCtables <- function()
     exon_start <- strsplitAsListOfIntegerVectors(ucsc_txtable$exonStarts)
     exon_end <- strsplitAsListOfIntegerVectors(ucsc_txtable$exonEnds)
     if (is.null(exon_count)) {
-        if (!identical(elementLengths(exon_start),
-                       elementLengths(exon_end)))
+        if (!identical(elementNROWS(exon_start),
+                       elementNROWS(exon_end)))
             stop("UCSC data anomaly: shape of 'ucsc_txtable$exonStarts' ",
                  "and 'ucsc_txtable$exonEnds' differ")
     } else {
-        if (!identical(elementLengths(exon_start), exon_count))
+        if (!identical(elementNROWS(exon_start), exon_count))
             stop("UCSC data anomaly: 'ucsc_txtable$exonStarts' ",
                  "inconsistent with 'ucsc_txtable$exonCount'")
-        if (!identical(elementLengths(exon_end), exon_count))
+        if (!identical(elementNROWS(exon_end), exon_count))
             stop("UCSC data anomaly: 'ucsc_txtable$exonEnds' ",
                  "inconsistent with 'ucsc_txtable$exonCount'")
     }
@@ -674,7 +685,8 @@ supportedUCSCtables <- function()
 
 .makeUCSCGenes <- function(genes, ucsc_txtable)
 {
-    #genes <- genes[genes$tx_name %in% ucsc_txtable$name, ]
+    #genes <- S4Vectors:::extract_data_frame_rows(genes,
+    #                             genes$tx_name %in% ucsc_txtable$name)
     genes
 }
 
@@ -703,8 +715,7 @@ supportedUCSCtables <- function()
                                                    circ_seqs)
     )
     oo <- order(rankSeqlevels(chrominfo[ , "chrom"]))
-    chrominfo <- chrominfo[oo, , drop=FALSE]
-    rownames(chrominfo) <- NULL
+    chrominfo <- S4Vectors:::extract_data_frame_rows(chrominfo, oo)
     message("OK")
     chrominfo
 }
@@ -724,7 +735,8 @@ getChromInfoFromUCSC <- function(genome,
 ### Prepare the 'metadata' data frame.
 ###
 
-.prepareUCSCMetadata <- function(genome, tablename, gene_id_type, full_dataset,
+.prepareUCSCMetadata <- function(genome, tablename, track, gene_id_type,
+                                 full_dataset,
                                  taxonomyId=NA, miRBaseBuild=NA)
 {
     message("Prepare the 'metadata' data frame ... ",
@@ -738,12 +750,16 @@ getChromInfoFromUCSC <- function(genome,
     }
         
     metadata <- data.frame(
-        name=c("Data source", "Genome", "Organism", "Taxonomy ID", "UCSC Table",
-               "Resource URL", "Type of Gene ID", "Full dataset",
+        name=c("Data source", "Genome", "Organism", "Taxonomy ID",
+               "UCSC Table", "UCSC Track",
+               "Resource URL", "Type of Gene ID",
+               "Full dataset",
                "miRBase build ID"),
         value=c("UCSC", genome, UCSCGenomeToOrganism(genome), taxonomyId,
-                tablename, "http://genome.ucsc.edu/", gene_id_type,
-                ifelse(full_dataset, "yes", "no"), miRBaseBuild)
+                tablename, track,
+                "http://genome.ucsc.edu/", gene_id_type,
+                ifelse(full_dataset, "yes", "no"),
+                miRBaseBuild)
     )
     message("OK")
     metadata
@@ -755,7 +771,7 @@ getChromInfoFromUCSC <- function(genome,
 ###
 
 .makeTxDbFromUCSCTxTable <- function(ucsc_txtable, genes,
-        genome, tablename, gene_id_type,
+        genome, tablename, track, gene_id_type,
         full_dataset,
         circ_seqs=DEFAULT_CIRC_SEQS,
         goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath",
@@ -771,7 +787,8 @@ getChromInfoFromUCSC <- function(genome,
             was not set (i.e. was set to '.')", sum(strand_is_dot))
         warning(paste(strwrap(msg, exdent=2), collapse="\n"))
         keep_idx <- which(!strand_is_dot)
-        ucsc_txtable <- ucsc_txtable[keep_idx, , drop=FALSE]
+        ucsc_txtable <- S4Vectors:::extract_data_frame_rows(ucsc_txtable,
+                                                            keep_idx)
     }
 
     transcripts <- .extractTranscriptsFromUCSCTxTable(ucsc_txtable)
@@ -779,8 +796,9 @@ getChromInfoFromUCSC <- function(genome,
                                                   transcripts$tx_id)
     genes <- .makeUCSCGenes(genes, ucsc_txtable)
     chrominfo <- .makeUCSCChrominfo(genome, circ_seqs, goldenPath_url)
-    metadata <- .prepareUCSCMetadata(genome, tablename, gene_id_type,
-                                     full_dataset, taxonomyId,  miRBaseBuild)
+    metadata <- .prepareUCSCMetadata(genome, tablename, track, gene_id_type,
+                                     full_dataset,
+                                     taxonomyId,  miRBaseBuild)
 
     message("Make the TxDb object ... ", appendLF=FALSE)
     txdb <- makeTxDb(transcripts, splicings, genes=genes,
@@ -856,7 +874,7 @@ makeTxDbFromUCSC <- function(genome="hg19",
         stop("GenomicFeatures internal error: invalid 'mapdef'")
     }
     .makeTxDbFromUCSCTxTable(ucsc_txtable, txname2geneid$genes,
-                             genome, tablename,
+                             genome, tablename, track,
                              txname2geneid$gene_id_type,
                              full_dataset=is.null(transcript_ids),
                              circ_seqs=circ_seqs,
@@ -865,5 +883,3 @@ makeTxDbFromUCSC <- function(genome="hg19",
                              miRBaseBuild=miRBaseBuild)
 }
 
-makeTranscriptDbFromUCSC <- function(...) .Defunct("makeTxDbFromUCSC")
-
